%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating VAR cofficients        %
%          subject to linear restrictions                        %
%                                                                %
%          Rmat*vec(Theta) = rvec                                %
%                                                                %
%          Function: linrest                                     %
%                                                                %
%          Input:                                                %
%                       y: nxT matrix                            %
%                       w: kxT matrix                            %
%                       Theta: nxk parameter matrix              %
%                       Rmat: qxnk matrix                        %
%                       rvec: qx1 vector                         %
%                       showwaitbar: integer (1=show)            %
%                       thisrec: indicator for recursions (0=no  %
%                          recursion, 1=recursion)               %
%                       use_rat: indicator for using rational    %
%                          basis for orthogonal matrix (0=no     %
%                          1=yes, the default)                   %
%                                                                %
%          Output:                                               %
%          		          theta: nxk matrix with restricted        %
%          		             estimates of parameters               %
%                       Sigma_theta: nkxnk matrix with estimated %
%                          asymptotic variances of theta         %
%                       infomat: 1x2 vector with information     %
%                          about iterations                      %
%                       abort: text string indicating if the     %
%                          cancel button was pressed or not      %
%                       inv_omega: inverse of older Omega matrix %
%                          (the one used for restricted          %
%                          estimate)                             %
%                       status: indicator if run was aborted     %
%                          or not                                %
%                                                                %
%          This version:        June 16, 2006                    %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [theta,Sigma_theta,infomat,abort,inv_omega,status] = linrest(y,w,Theta,Rmat,rvec,showwaitbar,thisrec,use_rat)

if nargin<6;
   showwaitbar = 0;
end;

if nargin<7;
   thisrec = 0;
end;

if nargin<8;
   use_rat = 0;
end;

curr_ini = find_inisettings;

if curr_ini.convcrit<10;
   ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
else;
   ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
end;
convcrit = mystr2num(ch_convcrit);
maxit = curr_ini.maxit;

if (curr_ini.extraccrit==1)&(showwaitbar==0)&(thisrec==0);
   maxit = 100;
   convcrit = 1.0e-004;
end;

status1 = 0;

%
% Preliminaries
%

epsilon = y-Theta*w;
[n T] = size(epsilon);
k = size(w,1);
Omega = (1/T)*epsilon*epsilon';
if rank(Omega,eps)<size(Omega,1);
   old_inv_Omega = eye(n);
else;
   old_inv_Omega = Omega\eye(n);
end;

if use_rat==1;
   H = null(Rmat,'r');
else;
   H = null(Rmat);
end;
h = (Rmat'/(Rmat*Rmat'))*rvec;

%
% If desired setup a waitbar informing on development
%

if showwaitbar==1;
   input_var.title = ['Progress for maximum of ' int2str(maxit) ' iterations'];
   input_var.facecolor = curr_ini.progress_facecolor;
   input_var.startfacecolor = curr_ini.progress_startfacecolor;
   input_var.edgecolor = curr_ini.progress_edgecolor;
   input_var.bgcolor = curr_ini.progress_bgcolor;
   input_var.stop = 1;
   input_var.clock = 1;
   input_var.name = 'ML Estimation under Linear Restrictions';
   mywait = progressdlg(0,input_var);
%   set(mywait,'Name','ML Estimation under Linear Restrictions');
   set(mywait,'Color',get(0,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;

%
% Starting up the iterations
%

grad = 1;
abort = '';
it = 1;
infomat = [it grad];
inv_Omega = old_inv_Omega;

while ((it<=maxit)&(convcrit<grad));
   if showwaitbar==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         progressdlg([it/maxit grad]);
      end;
   end;
   %
   % Estimating vec(theta) = H*vec(psi)+h
   %
   if nargout==6;
      check = (H'*kron(w*w',inv_Omega)*H);
      if rank(check)<size(check,1);
         status1 = 1;
         theta = [];
         break;
      end;
   end;
   vec_psi = (H'*kron(w*w',inv_Omega)*H)\((H'*vec(inv_Omega*y*w'))-(H'*kron(w*w',inv_Omega)*h));
   vec_theta = (H*vec_psi)+h;
   theta = vecinv(vec_theta,n);
   epsilon = y-theta*w;
   %
   % Check the inversion by computing the gradient based on the Omega
   % matrix used in the estimation
   %
   old_inv_Omega = inv_Omega;
   check = max(abs(H'*vec(inv_Omega*epsilon*w')));
   Omega = (1/T)*epsilon*epsilon';
   if rank(Omega,eps)<size(Omega,1);
      inv_Omega = eye(n);
   else;
      inv_Omega = Omega\eye(n);
   end;
   %
   % Adjust the gradient for the new Omega based on the precision of the
   % inversion.
   %
   grad = abs(max(abs(H'*vec(inv_Omega*epsilon*w')))-check);
   infomat = [it grad];
   it=it+1;
end;

%
% Computing the matrix with asymptotic variances
%

if status1==0;
   Sigma_theta = (H/(H'*kron(w*w',inv_Omega)*H))*H';
else;
   Sigma_theta = [];
end;

if showwaitbar==1;
   set(mywait,'UserData','done');
   close(mywait);
   drawnow;
end;

if nargout>=5;
   inv_omega = old_inv_Omega;
end;

if nargout==6;
   status = status1;
end;

%
% end of linrest.m
%
