%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for estimating cointegration          %
%          relations and obtaining eigenvalues                   %
%                                                                %
%          Function: mlcointbasic                                %
%                                                                %
%          Input:                                                %
%          		          Z0: nxT matrix                           %
%                       Z1: nq1xT matrix                         %
%                       Z2: n(p-1)+d matrix                      %
%                       r: cointegration rank                    %
%                                                                %
%          Output:                                               %
%          		          S_00: nxn matrix                         %
%          		          S_01: nxnq1 matrix                       %
%                       S_10: nq1xn matrix                       %
%                       S_11: nq1xnq1 matrix                     %
%                       M_02: nxn(p-1)+d matrix                  %
%                       M_22: n(p-1)+dxn(p-1)+d matrix           %
%                       M_12: nq1xn(p-1)+d matrix                %
%                       beta: nq1xr matrix                       %
%                       lambda: nx1 vector                       %
%                       m_trace: nx1 vector                      %
%                                                                %
%          This version:        November 5, 2004                 %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2004          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [S_00,S_01,S_10,S_11,M_02,M_22,M_12,beta,lambda,m_trace,beta_ort] = mlcointbasic(Z0,Z1,Z2,r)

%
% Determine the sample size
%

[n T] = size(Z0);
nq1 = size(Z1,1);

%
% Compute product moment matrices M_ij
%

M_00 = (1/T)*Z0*Z0';
M_01 = (1/T)*Z0*Z1';
if isempty(Z2)==0;
   M_02 = (1/T)*Z0*Z2';
   M_12 = (1/T)*Z1*Z2';
   M_22 = (1/T)*Z2*Z2';
else;
   M_02 = [];
   M_12 = [];
   M_22 = [];
end;
M_10 = M_01';
M_20 = M_02';
M_11 = (1/T)*Z1*Z1';
M_21 = M_12';

%
% Compute R(i,t) series
%

if isempty(Z2)==0;
   R0 = Z0-((M_02/M_22)*Z2);
   R1 = Z1-((M_12/M_22)*Z2);
else;
   R0 = Z0;
   R1 = Z1;
end;

%
% Compute product moment matrice S_ij
%

if isempty(Z2)==0;
   S_00 = M_00-(M_02/M_22)*M_20;
   S_11 = M_11-(M_12/M_22)*M_21;
   S_01 = M_01-(M_02/M_22)*M_21;
   S_10 = M_10-(M_12/M_22)*M_20;
else;
   S_00 = M_00;
   S_11 = M_11;
   S_01 = M_01;
   S_10 = M_10;
end;

%
% Solving the eigenvalue problem
%
% Step 1: Diagonalize S_11
%

[W,rho] = eig(S_11);
rho = diag(rho,0);
[rho2,AscInd] = sort(abs(rho));

nr = size(AscInd,1);
DescInd = zeros(nr,1);
i = 1;
while i<=nr;
   DescInd(i)=AscInd(nr+1-i);
   i =i+1;
end;

%
% Resort eigenvalues and eigenvectors
% in descending order
%

rho = rho(DescInd);
sqrho = sqrt(rho);
rho = diag(rho,0);
sqrho = diag(sqrho,0);
W = W(:,DescInd);

%
% Step 2: Reformulating eigenvalue problem to
%         calculating the eigenvalues for
%
%         adjS_11 = sqS_11*S_10*inverse(S_00)*S_01*sqS_11
%

sqS_11 = (W/sqrho)*W';

adjS_11 = sqS_11*(S_10/S_00)*S_01*sqS_11;

[U,lambda] = eig(adjS_11);

lambda = diag(lambda,0);
[lambda2,AscInd] = sort(abs(lambda));

nr = size(AscInd,1);
DescInd = zeros(nr,1);
i = 1;
while i<=nr;
   DescInd(i)=AscInd(nr+1-i);
   i =i+1;
end;

%
% Resort eigenvalues and eigenvectors
% in descending order
%

lambda = abs(lambda(DescInd));
U = U(:,DescInd);

%
% Renormalize U such that V'S_11V=I
%

V = sqS_11*U;

%
% Remove zero eigenvalues for the case when
% the number of variables i Z1 is greater than
% the number of variables in Z0 and adjust
% the eigenvectors accordingly
%

if n<nq1;
   lambda = lambda(1:n,:);
   V = V(:,1:n);
end;

m_trace = -T*log(ones(n,1)-lambda);
i=1;
while i<=n-1;
   m_trace(n-i,1)=m_trace(n-i,1)+m_trace(n-i+1,1);
   i = i+1;
end;

beta = V(:,1:r);

if nargout==11;
   beta_ort = V(:,r+1:n);
end;

%
% End of function mlcointbasic.m
%

