%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                %
%          MatLab function for recursive estimation of           %
%          cointegration relations under linear restrictions     %
%                                                                %
%          Function: recbetar                                    %
%                                                                %
%          Input:                                                %
%          	            S_00: nxn matrix                         %
%          	            S_01: nxnq1 matrix                       %
%                       S_10: nq1xn matrix                       %
%                       S_11: nq1xnq1 matrix                     %
%                       beta: nxr unrestricted cointegration     %
%                           vectors                              %
%                       t: the number of observations            %
%                       modelsdir: text string with path to the  %
%                           H_i.rest files with restrictions     %
%                           (can be a string with a space)       %
%                       modelfile: name of model                 %
%                       dcoint: indicator if constant or trend   %
%                           is in the cointegration space        %
%                                                                %
%          Output:                                               %
%          	            betar: nq1xr matrix with restricted      %
%          	               cointegration relations               %
%                       dfbeta: number of restrictions           %
%                          being imposed on beta                 %
%                       stdbeta: nq1xr matrix with estimated     %
%                          asymptotic conditional                %
%                          standard errors                       %
%                                                                %
%          This version:        April 21, 2006                   %
%          Written by: 	        Anders Warne                     %
%                               Copyright (c) 2001-2006          %
%                                                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [betar,dfbeta,stdbeta] = recbetar(S_00,S_11,S_01,S_10,beta,t,modelsdir,modelfile,dcoint,myalpha)

if nargin<10;
   myalpha = [];
end;

%
% Check if available matrices for restrictions are available
%

r = size(beta,2);
n = size(S_00,1);
nq1 = size(beta,1);
q1 = nq1-n-dcoint;
sivec = zeros(r,1);

i = 1;
while i<=r;
   file = strcat(modelsdir,'H_',num2str(i),'-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.rest');
   if fileexist(file)==1;
      h = loaddata(file);
      if i==1;
         H = h;
         bH = h;
      else;
         H = [H h];
         bH = [[bH zeros(size(bH,1),size(h,2))];[zeros(size(h,1),size(bH,2)) h]];
      end;
      sivec(i,1) = size(h,2);
   end;
   i = i+1;
end;

if sivec(r,1)==0;
   betar = 0;
   stdbeta = 0;
   newlambda = 0;
else;
   %
   % Computing input values for beta
   %
   S_bb = beta'*beta;
   %
   % Solving the eigenvalue problem
   %
   % Step 1: Diagonalize S_bb
   %
   [W,rho] = eig(S_bb);
   rho = diag(rho,0);
   [rho2,AscInd] = sort(abs(rho));
   nr = size(AscInd,1);
   DescInd = zeros(nr,1);
   i1 = 1;
   while i1<=nr;
      DescInd(i1)=AscInd(nr+1-i1);
      i1 =i1+1;
   end;
   %
   % Resort eigenvalues and eigenvectors
   % in descending order
   %
   rho = rho(DescInd);
   sqrho = sqrt(rho);
   rho = diag(rho,0);
   sqrho = diag(sqrho,0);
   W = W(:,DescInd);
   sqS_bb = W*inverse(sqrho)*W';
   oldlambda = zeros(r,1);
   i=1;
   dfbeta = 0;
   while i<=r;
      %
      % Selecting the restrictions from H
      %
      si = sivec(i,1);
      dfbeta = dfbeta+(nq1-r-si+1);
      sj = size(H,2);
      if i==1;
         Hsel = [eye(si);zeros(sj-si,si)];
      elseif i==r;
         Hsel = [zeros(sj-si,si);eye(si)];
      else;
         sk = sivec(1,1);
         j=2;
         while j<=i-1;
            sk = sk+sivec(j,1);
            j = j+1;
         end;
         Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
      end;
      Hi = H*Hsel;
      %
      % Step 2: Reformulating eigenvalue problem to
      %         calculating the eigenvalues for
      %
      %         adjS_bb = sqS_bb*beta'*Hi*inverse(Hi'*Hi)*Hi'*beta*sqS_bb
      %
      adjS_bb = sqS_bb*beta'*Hi*inverse(Hi'*Hi)*Hi'*beta*sqS_bb;
      [U,lambdai] = eig(adjS_bb);
      lambdai = diag(lambdai,0);
      [lambdai2,AscInd] = sort(abs(lambdai));
      nr = size(AscInd,1);
      DescInd = zeros(nr,1);
      i1 = 1;
      while i1<=nr;
         DescInd(i1)=AscInd(nr+1-i1);
         i1 =i1+1;
      end;
      %
      % Resort eigenvalues and eigenvectors
      % in descending order
      %
      lambdai = lambdai(DescInd);
      U = U(:,DescInd);
      %
      % Renormalize U such that V'S_bbV=I
      %
      V = sqS_bb*U;
      %
      % Computing the initial estimate of betai
      % using the first eigenvector from the
      % solution to the above eigenvalue problem
      %
      betai = beta*V(:,1);
      oldlambda(i,1) = lambdai(1,1);
      if i==1;
         betainit = betai;
      else;
         betainit = [betainit betai];
      end;
      i=i+1;
   end;
   betainit = real(betainit);
   %
   % The normalizations
   %
   file = strcat(modelsdir,'normalize-',num2str(n),'x',num2str(r),'x',num2str(dcoint+q1),'-',modelfile,'.norm');
   if fileexist(file)==1;
      h = loaddata(file);
   else;
      h = ones(r,1);
   end;
   %
   % Make sure that normalization works properly
   %
   % 1. check the length of h and fix if too short
   %
   if size(h,1)<r;
      hn = size(h,1)+1;
      for k=hn:r;
         si = sivec(k,1);
         sj = size(H,2);
         if k==1;
            Hsel = [eye(si);zeros(sj-si,si)];
         elseif k==r;
            Hsel = [zeros(sj-si,si);eye(si)];
         else;
            sk = sivec(1,1);
            for j=2:k-1;
               sk = sk+sivec(j,1);
            end;
            Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
         end;
         Hi = H*Hsel;
         for g=1:size(Hi,1);
            if abs(Hi(g,1))>1.0e-006;
               h = [h;g];
               g = size(Hi,1)+1;
            end;
         end;
      end;
   end;
   %
   % 2. check all coefficients to make sure they are valid
   %
   h = h(1:r,1);
   for k=1:r;
      si = sivec(k,1);
      sj = size(H,2);
      if k==1;
         Hsel = [eye(si);zeros(sj-si,si)];
      elseif k==r;
         Hsel = [zeros(sj-si,si);eye(si)];
      else;
         sk = sivec(1,1);
         for j=2:k-1;
            sk = sk+sivec(j,1);
         end;
         Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
      end;
      Hi = H*Hsel;
      mynorm = h(k,1);
      if max(abs(Hi(mynorm,:)))<1.0e-006;
         for l=1:r;
            if abs(Hi(l,1))>1.0e-006;
               h(k,1) = l;
               l = r+1;
            end;
         end;
      end;
   end;
   if dfbeta>0;
      %
      % Starting up the iterations
      %
      grad = 1;
      newsum = 1;
      curr_ini = find_inisettings;
      if curr_ini.convcrit<10;
         ch_convcrit = ['1.0e-00' num2str(curr_ini.convcrit)];
      else;
         ch_convcrit = ['1.0e-0' num2str(curr_ini.convcrit)];
      end;
      ccrit = mystr2num(ch_convcrit);

      it = 1;
      while ((it<=curr_ini.maxit)&(grad>ccrit));
         i = 1;
         newlambda = zeros(r,1);
         while i<=r;
            %
            % Selecting the restrictions from H
            %
            si = sivec(i,1);
            sj = size(H,2);
            if i==1;
               Hsel = [eye(si);zeros(sj-si,si)];
               tau = betainit(:,2:r);
            elseif i==r;
               Hsel = [zeros(sj-si,si);eye(si)];
               tau = betainit(:,1:r-1);
            else;
               sk = sivec(1,1);
               j=2;
               while j<=i-1;
                  sk = sk+sivec(j,1);
                  j = j+1;
               end;
               Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
               tau = [betainit(:,1:i-1) betainit(:,i+1:r)];
            end;
            Hi = H*Hsel;
            %
            % Computing the S_ij_tau matrices
            %
            S_00_tau = S_00-(S_01*tau*inverse(tau'*S_11*tau)*tau'*S_10);
            S_11_tau = S_11-(S_11*tau*inverse(tau'*S_11*tau)*tau'*S_11);
            S_01_tau = S_01-(S_01*tau*inverse(tau'*S_11*tau)*tau'*S_11);
            S_10_tau = S_10-(S_11*tau*inverse(tau'*S_11*tau)*tau'*S_10);
            HS_11_tau = Hi'*S_11_tau*Hi;
            %
            % Solving the eigenvalue problem for fixed tau
            %
            % Step 1: Diagonalize HS_11_tau
            %
            [W,rho] = eig(HS_11_tau);
            rho = diag(rho,0);
            [rho2,AscInd] = sort(abs(rho));
            nr = size(AscInd,1);
            DescInd = zeros(nr,1);
            i1 = 1;
            while i1<=nr;
               DescInd(i1)=AscInd(nr+1-i1);
               i1 = i1+1;
            end;
            %
            % Resort eigenvalues and eigenvectors
            % in descending order
            %
            rho = rho(DescInd);
            sqrho = sqrt(rho);
            rho = diag(rho,0);
            sqrho = diag(sqrho,0);
            W = W(:,DescInd);
            %
            % Step 2: Reformulating eigenvalue problem to
            %         calculating the eigenvalues for
            %
            %         adjHS_11_tau =
            %        sqHS_11_tau*Hi'*S_10_tau*inverse(S_00_tau)*S_01_tau*Hi*sqHS_11_tau
            %
            sqHS_11_tau = W*inverse(sqrho)*W';
            adjHS_11_tau = sqHS_11_tau*Hi'*S_10_tau*inverse(S_00_tau)*S_01_tau*Hi*sqHS_11_tau;
            [U,lambdai] = eig(adjHS_11_tau);
            lambdai = diag(lambdai,0);
            [lambdai2,AscInd] = sort(abs(lambdai));
            nr = size(AscInd,1);
            DescInd = zeros(nr,1);
            i1 = 1;
            while i1<=nr;
               DescInd(i1)=AscInd(nr+1-i1);
               i1 = i1+1;
            end;
            %
            % Resort eigenvalues and eigenvectors
            % in descending order
            %
            lambdai = lambdai(DescInd);
            U = U(:,DescInd);
            %
            % Renormalize U such that V'HS_11_tauV=I
            %
            V = sqHS_11_tau*U;
            newlambda(i,1) = lambdai(1,1);
            newbetai = Hi*V(:,1);
            %
            % Changing betainit to include newbetai
            % for column i
            %
            if i==1;
               betainit = [newbetai betainit(:,2:r)];
            elseif i==r;
               betainit = [betainit(:,1:r-1) newbetai];
            else;
               betainit = [betainit(:,1:i-1) newbetai betainit(:,i+1:r)];
            end;
            i = i+1;
         end;
         betainit = real(betainit);
         if curr_ini.betagrad==0;
            %
            % Calculate the convergence criterion using the
            % log-likelihood function
            %
            if min(ones(r,1)-oldlambda)<0.00000000000001;
               oldsum = 0;
            else;
               oldsum = sum(log(ones(r,1)-oldlambda));
            end;
            if min(ones(r,1)-newlambda)<0.00000000000001;
               newsum = r;
            else;
               newsum = sum(log(ones(r,1)-newlambda));
            end;
            logdetS_00 = log(det(S_00));
            grad = abs(((newsum-oldsum)/(logdetS_00+oldsum)));
         else;
            %
            % computing the gradient vector
            %
            if isempty(myalpha)==1;
               alphainit = S_01*betainit*inverse(betainit'*S_11*betainit);
               Omegainit = S_00-(S_01*betainit*inverse(betainit'*S_11*betainit)*betainit'*S_10);
            else;
               alphainit = myalpha;
               Omegainit = S_00-(alphainit*betainit'*S_11*betainit*alphainit');
            end;
            grad = t*vec(alphainit)'*kron(eye(r),inverse(Omegainit)*(S_01-(alphainit*betainit'*S_11)))*bH;
            grad = max(abs(grad));
         end;
         oldlambda = newlambda;
         it=it+1;
      end;
   else;
      newlambda = 0;
   end;
   betar = betainit;
   mycheck = 1;
   for i=1:r;
      normcoeffi = h(i,1);
      normbetai = betar(normcoeffi,i);
      if abs(normbetai)<1.0e-006;
         normbetai = 1;
         mycheck = 0;
      end;
      j=1;
      while j<=nq1;
         betar(j,i) = betar(j,i)/normbetai;
         j = j+1;
      end;
      i = i+1;
   end;
   %
   % Obtaining temporary estimates of alpha and Omega
   %
   alphar = S_01*betar*inverse(betar'*S_11*betar);
   Omegar = S_00-(S_01*betar*inverse(betar'*S_11*betar)*betar'*S_10);
   stdbeta = zeros(nq1,r);
   if mycheck==1;
      for i=1:r;
         %
         % Selecting the restrictions from H
         %
         si = sivec(i,1);
         if si>=2;
            sj = size(H,2);
            if i==1;
               Hsel = [eye(si);zeros(sj-si,si)];
            elseif i==r;
               Hsel = [zeros(sj-si,si);eye(si)];
            else;
               sk = sivec(1,1);
               for j=2:i-1;
                  sk = sk+sivec(j,1);
               end;
               Hsel = [zeros(sk,si);eye(si);zeros(sj-si-sk,si)];
            end;
            Hi = H*Hsel;
            %
            % Determining the normalizations
            %
            normelem = h(i,1);
            myexit = 0;
            for j=1:si;
               %
               % Fix Hi to make sure that the normalizing row
               % contains one unit element and zeros
               %
               normmatrix = Hi(normelem,:);
               for k=1:(n+q1+dcoint);
                  if k~=normelem;
                     checkHi = [normmatrix;Hi(k,:)];
                     if rank(checkHi*checkHi')==size(checkHi,1);
                        normmatrix = checkHi;
                        if size(normmatrix,1)==size(Hi,2);
                           break;
                        end;
                     end;
                  end;
               end;
               Hi = Hi/normmatrix;
               check = Hi(normelem,j);
               if (abs(check)>0)&(check~=1);
                  Hi(:,j) = (1/Hi(normelem,j))*Hi(:,j);
                  check = Hi(normelem,j);
               end;          
               if check==1;
                  %
                  % Split Hi into hi and H_i
                  %
                  if j==1;
                     hi = Hi(:,1);
                     H_i = Hi(:,2:si);
                  elseif j==si;
                     hi = Hi(:,si);
                     H_i = Hi(:,1:si-1);
                  else;
                     hi = Hi(:,j);
                     H_i = [Hi(:,1:j-1) Hi(:,j+1:si)];
                  end;
                  %
                  % Check H_i to make sure it is of full rank
                  %
                  check2 = H_i'*H_i;
                  if abs(det(check2))>=.000000001;
                     myexit=1;
                  end;
               end;
               if myexit==1;
                  break;
               end;
            end;
            %
            % Computing the asymptotic covariance for the
            % si-1 free parameters of column i in betar
            %
            information = alphar(:,i)'*inverse(Omegar)*alphar(:,i)*H_i'*S_11*H_i;
            covbeta = (1/t)*H_i*inverse(information)*H_i';
            %
            % Computing the asyptotic standard errors and
            % including them in stdbeta
            %
            stdbeta(:,i) = sqrt(abs(diag(covbeta)));
         end;
      end;
   end;
end;

%
% end of recbetar.m
%
